﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using System.Linq;
using Utils;
using System.Threading;

namespace Reptile
{
    class Program
    {
        static Dictionary<string, List<DataItem>> archiveList = new Dictionary<string, List<DataItem>>();//文章列表
        static List<DataItem> categoryList = new List<DataItem>();//分类列表
        static string userId = "hlxs";//用户ID，根据这个ID获取这个人的博客目录
        const string archive = "archive";
        const string category = "category";

        private static Stream GetStream(int timeout, string url, string param)
        {
            try
            {
                if (!url.StartsWith("http://", StringComparison.OrdinalIgnoreCase))
                {
                    url = "http://" + url;
                }
                HttpWebRequest req = (HttpWebRequest)HttpWebRequest.Create(url);
                if (!string.IsNullOrEmpty(param))
                {
                    req.Method = "POST";
                    req.ContentType = "application/x-www-form-urlencoded";
                    byte[] bs = Encoding.ASCII.GetBytes(param);
                    req.ContentLength = bs.Length;
                    using (Stream reqStream = req.GetRequestStream())
                    {
                        reqStream.Write(bs, 0, bs.Length);
                    }
                }

                HttpWebResponse HttpWResp = (HttpWebResponse)req.GetResponse();
                return HttpWResp.GetResponseStream();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return null;
            }
        }

        //获取页面的内容
        private static string GetContent(string url, string param)
        {
            using (Stream myStream = GetStream(24000, url, param))
            {
                if (myStream == null)
                {
                    return string.Empty;
                }
                using (StreamReader sr = new StreamReader(myStream, Encoding.UTF8))
                {
                    if (sr.Peek() > 0)
                    {
                        return sr.ReadToEnd();
                    }
                }
                return string.Empty;
            }
        }

        static MatchCollection Filter(string url, string param)
        {
            string content = GetContent(url, param);
            if (string.IsNullOrEmpty(content))
            {
                return null;
            }
            string regex = @"<a(?:(?!href=).)*href=(['""]?)(?<url>[^""\s>]*)\1[^>]*>(?<text>(?:(?!</?a\b).)*)</a>";
            Regex rx = new Regex(regex);
            return rx.Matches(content);
        }

        //获取文章
        static void GetArchive(string url, string param)
        {
            MatchCollection matchs = Filter(url, param);
            if (matchs == null)
            {
                return;
            }
            foreach (Match m in matchs)
            {
                string curUrl = m.Groups["url"].Value;
                if (curUrl.IndexOf(userId + "/" + archive) >= 0 && curUrl.IndexOf('#') < 0)
                {
                    DataItem item = new DataItem { Text = m.Groups["text"].Value, Value = curUrl };
                    if (!archiveList.ContainsKey(url))
                    {
                        archiveList.Add(url, new List<DataItem> { item });
                    }
                    else
                    {
                        if (archiveList[url].FirstOrDefault(archiveItem => archiveItem.Value == curUrl) == null)
                        {
                            archiveList[url].Add(item);
                        }
                    }
                }
            }
            DataItem curCategory = categoryList.FirstOrDefault(m => m.Value == url);
            if (curCategory != null)
            {
                Console.WriteLine("    " + curCategory.Text);
            }
        }

        //获取文章分类
        static bool GetCategory(string url, string param)
        {
            MatchCollection matchs = Filter(url, param);
            if (matchs == null)
            {
                return false;
            }
            Console.Clear();
            Console.WriteLine("已完成您的博客分类：");
            foreach (Match m in matchs)
            {
                if (m.Groups["url"].Value.IndexOf(userId + "/" + category) >= 0)
                {
                    categoryList.Add(new DataItem { Text = m.Groups["text"].Value, Value = m.Groups["url"].Value });
                }
            }
            return categoryList.Count == 0 ? false : true;
        }

        static void Main(string[] args)
        {
            Console.WriteLine("请输入你的博客园ID后回车：");
            userId = Console.ReadLine();
            string url = "http://www.cnblogs.com/" + userId + "/mvc/blog/sidecolumn.aspx";//获取博客分类地址
            string param = "blogApp=" + userId;//获取你的博客分类需要你的ID
            Console.WriteLine("正在连接服务器，请稍候...");
            bool isOK= GetCategory(url, param);//获取博客分类
            if (!isOK)
            {
                Console.Clear();
                Console.WriteLine("你输入的博客园ID不正确，系统将在10秒后爆炸，请注意安全!");
                Thread.Sleep(5000);
                for (int i = 5; i >= 1; i--)
                {
                    Thread.Sleep(1000);
                    Console.Clear();
                    Console.WriteLine(i);
                }
                Environment.Exit(0);
            }
            foreach (DataItem item in categoryList)
            {
                GetArchive(item.Value, string.Empty);//获取分类中的博客
            }
            Console.Clear();
            CreateDir();
            Console.Read();
            
        }

        //生成博客目录
        static void CreateDir()
        { 
            IOHelper.DeleteFile(Environment.CurrentDirectory + "/我的博客目录.txt");
            string divFormat = "<div style='width:100%;float:left;margin-top:20px;background-color:#999999;font-size:20px;color:White;padding-left:5px'>{0}</div>";
            string aFormat = "<a href='{0}' target='_blank' title='{1}'>{1}</a>";
            string liFormat = "<li style='width:49%;float:left;line-height:30px;'>{0}</li>";

            string[] catelist = { "算法", "智力题", "C++", "读书", "分析", "C#", "Windows" };//这些排在前面
            for (int i = catelist.Length - 1; i >= 0; i--)
            {
                DataItem item = categoryList.FirstOrDefault(m => m.Text.Contains(catelist[i]));
                if (item != null)
                {
                    categoryList.Remove(item);
                    categoryList.Insert(0, item);
                }
            }

            foreach (DataItem categoryItem in categoryList)
            {
                Console.WriteLine(categoryItem.Text);
                WriteLine(string.Format(divFormat, categoryItem.Text));
                List<DataItem> list;
                if (archiveList.TryGetValue(categoryItem.Value, out list))
                {
                    WriteLine("<ul style='padding-top:10px;clear:both;list-style:none;'>");
                    foreach (DataItem archiveItem in list)
                    {
                        Console.WriteLine("\t" + archiveItem.Text);
                        WriteLine(string.Format(liFormat, string.Format(aFormat, archiveItem.Value, archiveItem.Text)));
                    }
                    WriteLine("</ul>");
                    Console.WriteLine();
                }
            }
            WriteLine("<div style='clear:both;'>");
        }

        private static void WriteLine(string content)
        {
            IOHelper.WriteLine(Environment.CurrentDirectory + "/我的博客目录.txt", content);
        }
    }
}
